/*
 *  rtfutils.c
 * utilities for RTF-to-LaTeX2e translation writer code.
 * (c) 2003 Richard Bornat
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "rtfutils.h"

void _verbose (char *fmt, ...) {
    va_list args;

    va_start (args,fmt);
    vprintf (fmt, args);
    va_end (args);
    fputs("\n", stdout); fflush(stdout);
}

void _verbose_wonl (char *fmt, ...) {
    va_list args;

    va_start (args,fmt);
    vprintf (fmt, args);
    va_end (args);
    fflush(stdout);
}

void _unverbose (void *fmt, ...) {
    return;
}

/*-*
(This was __cole_dump)

@func (void) _verbose_dump dump the content of memory
@param (void *) m memory position which content will be dumped
@param (void *) start memory position from where calculate
offset
@param (int) length size in bytes of the dumped memory
@param (char *) msg optional message, can be NULL
*-*/

void _verbose_dump (void *_m, void *_start, int length, char *msg)
{
    unsigned char * pm;
    char buff[18];
    long achar;
    unsigned char * m;
    unsigned char * start;

    if (_m == NULL) {
        printf ("VERBOSE: can't dump because m is NULL\n");
        return;
    }
    if (_start == NULL) {
        printf ("VERBOSE: can't dump because start is NULL\n");
        return;
    }

    m = (unsigned char *)_m;
    start = (unsigned char *)_start;
    buff[8] = '-';
    buff[17] = 0;
    if (msg != NULL)
        printf ("VERBOSE: %s (from 0x%08x length 0x%08x (%d)):\n",
                msg, m - start, length, length);
    for (pm = m; pm - m < length; pm++) {
        achar = (pm - m) % 16;
        /* print offset */
        if (achar == 0)
            printf ("%08x  ", (pm - m) + (m - start));
        /* write char in the right column buffer */
        buff[achar + (achar < 8 ? 0 : 1)] = (isprint (*pm) ? *pm : '.');
        /* print next char */
        if (! ((pm - m + 1) % 16))
            /* print right column */
            printf ("%02x  %s\n", *pm, buff);
        else if (! ((pm - m + 1) % 8))
            printf ("%02x-", *pm);
        else
            printf ("%02x ", *pm);
    }
    achar = (pm - m) % 16;
    if (achar) {
        int i;
        for (i = 0; i < (16 - achar) * 3 - 1; i++)
            printf (" ");
        if (achar != 8)
            buff [achar] = 0;
        printf ("  %s\n", buff);
    }
}

/* vasprintf and asprintf aren't part of the ISO standard (1999).  Yah boo!
But vsnprintf and snprintf are ... so we fabricate facsimiles.
RB
*/
static int _vasp(char **buf, char *fmt, va_list args) {
    /* in the way of these things, the C standard is very confusing about va_copy.
    It seems to flatly prohibit the example which it displays.  I think they meant
    to say dest, not ap. I hope this code gets round all the problems.
    RB
    */
    size_t bufsiz=0, n=0;
    *buf = NULL;
    
    while (n>=bufsiz) {
        va_list copy;
	
	if (*buf!=NULL)
	    free(*buf);
        bufsiz = n+1;
        if ((*buf = (char*)malloc(bufsiz))==NULL)
            return -1;
        __va_copy(copy, args);
        n = vsnprintf(*buf, bufsiz, fmt, copy);
        va_end(copy);
    }

    return n;
}

extern int _asprintf(char **buf, char *fmt, ...) {
    va_list args;
    int ret;
    
    va_start (args,fmt);
    ret = _vasp (buf, fmt, args);
    va_end(args);
    return ret;
}

extern int _vasprintf(char **buf, char *fmt, va_list args) {
    return _vasp(buf, fmt, args);
}
